"""
Git Clone & Pull with CI-Pipeline

Linux: Disable History <unset HISTFILE>
"""
import os
import functools
import yaml
from .command import command
from .pipeline import run_pipeline


def load_auth(file_name: str):
    """Load the credentials from a YAML file"""

    with open(file_name, "r") as f:
        pipeline = yaml.safe_load(f)
        return pipeline


def git_base(
    append: str = None,
    method: str = None,
    url: str = None,
    username: str = None,
    password: str = None,
    path: str = None,
    shell: bool = False,
):
    """Get Git from Https"""
    text = ""
    secure = False

    # Clean
    if url.startswith("https"):
        secure = True
    if secure:
        repo = url.replace("https://", "")
    else:
        repo = url.replace("http://", "")

    # Build
    if secure:
        text = f"git {method} https://{username}:{password}@{repo} {path or ''}".strip()
    else:
        text = f"git {method} http://{username}:{password}@{repo} {path or ''}".strip()

    # os.system(cmd_text + "; history -d $(history 1)")
    cmd_text = f"{append or ''} " + text
    if shell:
        return command(cmd_text)
    else:
        os.system(cmd_text)


def git_run(
    method: str = "clone",
    file_name: str = ".ci-auth.yml",
    go_to: bool = False,
    shell: bool = False,
):
    """Load setup from a file"""
    config = load_auth(file_name)
    path = None
    append = None
    if method == "clone":
        path = config.get("path")
    if go_to:
        append = f'cd {config.get("path")} &&'
        shell = True
    return git_base(
        method=method,
        url=config.get("url"),
        username=config.get("username"),
        password=config.get("password"),
        path=path,
        append=append,
        shell=shell,
    )


def pull(file_name: str = ".ci-auth.yml"):
    """Git Pull"""
    do_pull = functools.partial(git_run, method="pull", file_name=file_name, go_to=True)
    log = do_pull()
    if "Already up to date" not in log:
        run_pipeline()


def clone(file_name: str = ".ci-auth.yml"):
    """Git Clone"""
    do_clone = functools.partial(
        git_run, file_name=file_name, method="clone", shell=True
    )
    log = do_clone()
    if "$ERROR" not in log:
        run_pipeline()



import yaml
import subprocess


def run_commands(commands):
    """Run All Scripts"""
    for command in commands:
        subprocess.call(command, shell=True)


def enforce_list(script):
    """Script(s) as List"""
    script = script or []
    if isinstance(script, str):
        script = [script]
    return script


def collect_jobs_by_stage(file_name: str):
    """Group Jobs By Stage"""

    jobs_by_stage = {}

    # Load the pipeline definition from a YAML file
    with open(file_name, "r") as f:
        pipeline = yaml.safe_load(f)

    # Group the jobs by stage
    for job in pipeline:
        if job not in ["stages"]:
            obj = pipeline[job]
            obj["name"] = job
            stage = obj["stage"]
            if stage not in jobs_by_stage:
                jobs_by_stage[stage] = []
            jobs_by_stage[stage].append(obj)
    return jobs_by_stage


def run_pipeline(
    stages: list | None = None, file_name: str = ".ci-git.yml", break_length: int = 45
):
    """CI/CD Pipeline"""

    jobs_by_stage = collect_jobs_by_stage(file_name)

    do_check = False
    if stages:
        do_check = True

    # Loop through each stage and execute the jobs in order
    for stage, jobs in jobs_by_stage.items():
        if do_check and stage not in stages:
            continue
        else:
            stage_text = f"Stage: {stage}"
            print("-" * break_length)
            print(stage_text)
            print("-" * break_length)
            for job in jobs:
                print(f'Executing job: {job["name"]}')
                print("-" * break_length)
                script = job.get("script", [])
                script = enforce_list(script)
                run_commands(script)
                print(f'Job completed: {job["name"]}', end="\n\n")

    # Done
    print("=" * (break_length))
    print("All jobs completed.")



import subprocess


def command(command: str):
    """Run Shell Command"""
    result = subprocess.run(command, stdout=subprocess.PIPE, shell=True)
    output = result.stdout.decode("utf-8")
    if result.returncode != 0:
        output = "$ERROR"
    return output



import argparse
from .pipeline import run_pipeline as run
from .gitman import clone, pull

parser = argparse.ArgumentParser()

# Args
parser.add_argument("action", help="Options: [run, clone, pull]")
parser.add_argument("-s", "--stages", help="Stages of the Pipeline you want to run")
parser.add_argument("-c", "--config", help="Use custom setting's file")

args = parser.parse_args()

match args.action:
    case "run":
        run()
    case "clone":
        clone()
    case "pull":
        pull()
